##########################################
#Author: Kishan Silva Ratnayake (ratki01)
#Created on: 06.03.2013
#Updated on: 10.10.2014
#
#Version:10.10.2014
#PM Supported versions - up to PM 2.3.4
##########################################

use strict;
use warnings;
use REST::Client;
use Path::Class;
use XML::Tidy;
use autodie; # die if problem reading or writing a file

if(scalar(@ARGV) < 1)
{
        
        print "\nUsage: perl bulk_mf_vc_export.pl <DA_HOST>:<PORT>\n";
        print "\n<DA_HOST> - Data Aggregator hostname/ip\n";
        print "<PORT> - Data Aggregator port - default is 8581\n";
        print "\nExample:\n";
        print "perl bulk_mf_vc_export.pl IM-DA:8581 \n\n";
        exit 1;
}

my $DA_URL = $ARGV[0];


#make connection to DA web service
my $client = REST::Client->new();
$client->setHost('http://' . $DA_URL);
$client->addHeader('Accept', 'application/xml');

my $os_type;
use Cwd 'abs_path';
#abs_path($0) =~ m/(.*\/)(.*\.pl)/;
abs_path($0) =~ m/(.*(\/|\\))(.*\.pl)/;
my $current_working_dir = $1;
if($^O =~ m/Win/)
{
	$current_working_dir =~ s/\//\\/g;
	$os_type = 'W';
}
else
{
        $os_type = 'L';
}

#read in all interface aliases and speeds that need to be updated
#my $dir = dir($current_working_dir);
my $dir = dir($current_working_dir);
my $file = $dir->file('mfs_vcs_to_export.txt');
my $content = $file->slurp();
my $file_handle = $file->openr();

my @vendor_certs;
my @metric_fams;
my $vc_set = 0;
my $mf_set = 0;
my $vc_count = 0;
my $mf_count = 0;

while( my $line = $file_handle->getline() ) 
{

        if($line =~ m/^#VC/)
        {
            $mf_set = 0;
            $vc_set = 1;
        }
         
        if($line =~ m/^#MF/)
        {
            $mf_set = 1;
            $vc_set = 0;
        }  
        
        if(($mf_set == 1) && ($line !~ m/^$/) && ($line !~ m/^#MF/))
        {
            $line=~ s/^\s+//; 
            $line=~ s/\s+$//;
            $metric_fams[$mf_count++] = $line;
        } 
        
        if(($vc_set == 1) && ($line !~ m/^$/) && ($line !~ m/^#VC/))
        {
            $line=~ s/^\s+//; 
            $line=~ s/\s+$//;
            $vendor_certs[$vc_count++] = $line;
        }
}

#print @vendor_certs;

my $mf_top = '<DataModel namespace="http://im.ca.com/normalizer" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="IMDBCertificationFacet.xsd">';
my $vc_top = '<DataModel namespace="http://im.ca.com/certifications/snmp" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="SNMPCertificationFacet.xsd">';
my $bottom = '</DataModel>';
print "Please be patient...\n";
if($mf_count > 0)
{
    print "Working on Metric Family export...\n";
    my @all_metric_fams;
    $client->GET('/typecatalog/metricfamilies');
    my $x = 0;
    if($client->responseCode() eq '200')
    {
        
        my $tidy_obj = XML::Tidy->new( xml => $client->responseContent());
        $tidy_obj->tidy();
        $tidy_obj->write($current_working_dir.'all_metric_fams.txt'); 
        $file = $dir->file('all_metric_fams.txt');
        $content = $file->slurp();
        $file_handle = $file->openr();
        while( my $line = $file_handle->getline() ) {
               
               $line=~ s/^\s+//; 
               $line=~ s/\s+$//;
               $all_metric_fams[$x++] = $line;
        }
    }
    
    for($x=0;$x<scalar(@metric_fams);$x++)
    {
        my $current_metric_fam = $metric_fams[$x];
        my $temp_metric_fam = '';
        my $matched = 0;
        my $mf_def = '';
        my $y = 0;
        for($y=0;$y<scalar(@all_metric_fams);$y++)
        {
            
            
            if(($all_metric_fams[$y] !~ m/\<DataModel namespace.*/) && ($all_metric_fams[$y] !~ m/\<\?xml version="1.0" encoding="UTF-8"\?\>/) && ($all_metric_fams[$y] !~ m/\<\!--Auto-generated by the type catalog local manager\.--\>/))
            {
                $mf_def = $mf_def . $all_metric_fams[$y] . "\n"; 
            }
            
            if($all_metric_fams[$y] =~ m/\<DisplayName\>(.*)\<\/DisplayName\>/)
            {
                $temp_metric_fam = $1;
                $temp_metric_fam=~ s/^\s+//; 
                $temp_metric_fam=~ s/\s+$//;
                if($current_metric_fam eq $temp_metric_fam)
                {
                        $matched = 1;
                }
            }
            
            
            if($all_metric_fams[$y] =~ m/\<\/FacetType\>/)
            {
                if($matched == 1)
                {
                    $mf_def = $mf_top . "\n" . $mf_def . $bottom;
                    my $directory;
                    if($os_type eq 'W')
                    {
                       $directory = $current_working_dir . '\mf';
                    }
                    else
                    {
                       $directory = $current_working_dir . '/mf';     
                    }
                    
	            unless(-e $directory or mkdir $directory) {
                    die "Unable to create $directory\n";
                    }
                    
                    if($os_type eq 'W')
                    {       
                        open (MYFILE, '>'. $directory . '\\' . $temp_metric_fam . '.xml');
                    }
                    else
                    {
                        open (MYFILE, '>'. $directory . '/' . $temp_metric_fam . '.xml');
                    }
                    print MYFILE $mf_def;
                    close(MYFILE);
                    $y = scalar(@all_metric_fams);
                    $metric_fams[$x] = 'DONE';
                }
                
                $mf_def = '';
                
            }
            
           
        }
    }
}

if($vc_count > 0)
{
    print "Working on Vendor Certificate export...\n";
    my @all_vendor_certs;
    $client->GET('/typecatalog/certifications/snmp');
    my $x = 0;
    if($client->responseCode() eq '200')
    {
        my $tidy_obj = XML::Tidy->new( xml => $client->responseContent());
        $tidy_obj->tidy();
        $tidy_obj->write($current_working_dir.'all_vendor_certs.txt');
        $file = $dir->file('all_vendor_certs.txt');
        $content = $file->slurp();
        $file_handle = $file->openr();
        while( my $line = $file_handle->getline() ) {
               
               $line=~ s/^\s+//; 
               $line=~ s/\s+$//;
               $all_vendor_certs[$x++] = $line;
        }
    }
    
    for($x=0;$x<scalar(@vendor_certs);$x++)
    {
        my $current_vendor_cert = $vendor_certs[$x];
        my $y = 0;
        my $vc_def = '';
        my $temp_vendor_cert = '';
        my $matched = 0;
        for($y=0;$y<scalar(@all_vendor_certs);$y++)
        {
            if(($all_vendor_certs[$y] !~ m/\<DataModel namespace.*/) && ($all_vendor_certs[$y] !~ m/\<\?xml version="1.0" encoding="UTF-8"\?\>/) && ($all_vendor_certs[$y] !~ m/\<\!--Auto-generated by the type catalog local manager\.--\>/))
            {
                $vc_def = $vc_def . $all_vendor_certs[$y] . "\n"; 
            }
            
            if($all_vendor_certs[$y] =~ m/\<DisplayName\>(.*)\<\/DisplayName\>/)
            {
                $temp_vendor_cert = $1;
                if($current_vendor_cert eq $temp_vendor_cert)
                {
                        $matched = 1;
                }
            }
            
            
            if($all_vendor_certs[$y] =~ m/\<\/FacetType\>/)
            {
                if($matched == 1)
                {
                    $vc_def = $vc_top . "\n" . $vc_def . $bottom;
                    my $directory;
                    
                    if($os_type eq 'W')
                    {
                       $directory = $current_working_dir . '\vc';
                    }
                    else
                    {
                       $directory = $current_working_dir . '/vc';     
                    }
                    
	            unless(-e $directory or mkdir $directory) {
                    die "Unable to create $directory\n";
                    }
                    
                    if($os_type eq 'W')
                    {       
                        open (MYFILE, '>'. $directory . '\\' . $temp_vendor_cert . '.xml'); 
                    }
                    else
                    {
                        open (MYFILE, '>'. $directory . '/' . $temp_vendor_cert . '.xml');
                    }
                    print MYFILE $vc_def;
                    close(MYFILE);
                    $y = scalar(@all_vendor_certs); 
                    $vendor_certs[$x] = 'DONE'
                      
                }
                
                $vc_def = '';
                
            }
        }
        
    }
    
}


if($os_type eq 'W')
{       
    open (MYFILE, '>'.$current_working_dir.'\export_summary.txt');
}
else
{
    open (MYFILE, '>'.$current_working_dir.'/export_summary.txt');
}

print MYFILE "Summary of exporting Metric Families and Vendor Certificates\n";
print MYFILE "============================================================\n\n\n";
my $x = 0;
if($mf_count > 0)
{
    my $success = 0;
    for($x=0;$x<scalar(@metric_fams);$x++)
    {
        if($metric_fams[$x] eq 'DONE')
        {
                $success++;
        }
    }
     
    print MYFILE "Total number of Metric Families to be exported        : " . scalar(@metric_fams) . "\n";
    print MYFILE "Total number of Metric Families successfully exported : " . $success . "\n\n";
    print MYFILE "List of Metric Families not exported\n";
    print MYFILE "---------------------------------\n";
    for($x=0;$x<scalar(@metric_fams);$x++)
    {
        if($metric_fams[$x] ne 'DONE')
        {
                print MYFILE $metric_fams[$x] . "\n";
        }
    }
    print MYFILE "\n\n\n";
}
else
{
    print MYFILE "There were no Metric Families to export.\n\n";
}


if($vc_count > 0)
{
    my $success = 0;
    for($x=0;$x<scalar(@vendor_certs);$x++)
    {
        if($vendor_certs[$x] eq 'DONE')
        {
                $success++;
        }
    }
     
    print MYFILE "Total number of Vendor Certs to be exported        : " . scalar(@vendor_certs) . "\n";
    print MYFILE "Total number of Vendor Certs successfully exported : " . $success . "\n\n";
    print MYFILE "List of Vendor Certs not exported\n";
    print MYFILE "---------------------------------\n";
    for($x=0;$x<scalar(@vendor_certs);$x++)
    {
        if($vendor_certs[$x] ne 'DONE')
        {
                print MYFILE $vendor_certs[$x] . "\n";
        }
    }
}
else
{
    print MYFILE "There were no Vendor Certs to export.\n\n";
}

print "\n\nSummary of results have been logged to the following: \n" . $current_working_dir . "export_summary.txt\n\n";
#print "mf and vc directories containing exported Metric Families and Vendor Certs have been created here: " . $current_working_dir;
if($vc_count > 0 || $mf_count > 0)
{
    print "\nThe following directories have been created:\n";
    if($mf_count > 0)
    {
        print "-Metric Families exported are located here: \n" . $current_working_dir . 'mf';
        print "\n";
    }
    if($vc_count > 0)
    {
        print "-Vendor Certificates exported are located here: \n" . $current_working_dir . "vc";
        print "\n";
    }
}
close(MYFILE);